@@ -1,7 +1,5 @@
-from __future__ import print_function, absolute_import, division
-
-from numba import config
 import numba.testing
+from numba.core import config
 
 if config.ENABLE_CUDASIM:
     from .simulator_init import *
@@ -9,6 +7,7 @@ else:
     from .device_init import *
     from .device_init import _auto_device
 
+from numba.cuda.compiler import compile_ptx, compile_ptx_for_current_device
 
 def test(*args, **kwargs):
     if not is_available():
