@@ -1,7 +1,6 @@
-from __future__ import print_function, absolute_import, division
 from ctypes import *
 
-from . import _extras
+from numba.cuda.cudadrv import _extras
 
 cu_device = c_int
 cu_device_attribute = c_int     # enum
@@ -20,6 +19,10 @@ cu_ipc_mem_handle = (c_byte * _extras.CUDA_IPC_HANDLE_SIZE)   # 64 bytes wide
 
 cu_occupancy_b2d_size = CFUNCTYPE(c_size_t, c_int)
 
+# See https://docs.nvidia.com/cuda/cuda-driver-api/group__CUDA__TYPES.html
+CU_STREAM_DEFAULT = 0
+CU_STREAM_LEGACY = 1
+CU_STREAM_PER_THREAD = 2
 
 API_PROTOTYPES = {
 # CUresult cuInit(unsigned int Flags);
