@@ -1,9 +1,8 @@
-from __future__ import print_function, absolute_import, division
 import itertools
 import llvmlite.llvmpy.core as lc
 from .cudadrv import nvvm
 from .api import current_context
-from numba import cgutils
+from numba.core import cgutils
 
 
 def declare_atomic_cas_int32(lmod):
