@@ -1,21 +1,19 @@
-from __future__ import print_function, absolute_import
 import re
 from llvmlite.llvmpy.core import (Type, Builder, LINKAGE_INTERNAL,
                        Constant, ICMP_EQ)
 import llvmlite.llvmpy.core as lc
 import llvmlite.binding as ll
 
-from numba import typing, types, cgutils, debuginfo, dispatcher
-from numba.utils import cached_property
-from numba.targets.base import BaseContext
-from numba.targets.callconv import MinimalCallConv
-from numba.targets import cmathimpl
-from numba.typing import cmathdecl
+from numba.core import typing, types, dispatcher, debuginfo, itanium_mangler, cgutils
+from numba.core.utils import cached_property
+from numba.core.base import BaseContext
+from numba.core.callconv import MinimalCallConv
+from numba.core.typing import cmathdecl
 
-from numba import itanium_mangler
 from .cudadrv import nvvm
-from . import codegen, nvvmutils
+from numba.cuda import codegen, nvvmutils
 from .decorators import jitdevice
+from numba.cpython import cmathimpl
 
 
 # -----------------------------------------------------------------------------
@@ -87,6 +85,20 @@ class CUDATargetContext(BaseContext):
     def target_data(self):
         return self._target_data
 
+    @cached_property
+    def nonconst_module_attrs(self):
+        """
+        Some CUDA intrinsics are at the module level, but cannot be treated as
+        constants, because they are loaded from a special register in the PTX.
+        These include threadIdx, blockDim, etc.
+        """
+        from numba import cuda
+        nonconsts = ('threadIdx', 'blockDim', 'blockIdx', 'gridDim', 'laneid',
+                     'warpsize')
+        nonconsts_with_mod = tuple([ (types.Module(cuda), nc)
+                                    for nc in nonconsts ])
+        return nonconsts_with_mod
+
     @cached_property
     def call_conv(self):
         return CUDACallConv(self)
@@ -186,15 +198,48 @@ class CUDATargetContext(BaseContext):
         wrapfn = library.get_function(wrapfn.name)
         return wrapfn
 
-    def make_constant_array(self, builder, typ, ary):
+    def make_constant_array(self, builder, aryty, arr):
         """
-        Return dummy value.
-
-        XXX: We should be able to move cuda.const.array_like into here.
+        Unlike the parent version.  This returns a a pointer in the constant
+        addrspace.
         """
 
-        a = self.make_array(typ)(self, builder)
-        return a._getvalue()
+        lmod = builder.module
+
+        constvals = [
+            self.get_constant(types.byte, i)
+            for i in iter(arr.tobytes(order='A'))
+        ]
+        constary = lc.Constant.array(Type.int(8), constvals)
+
+        addrspace = nvvm.ADDRSPACE_CONSTANT
+        gv = lmod.add_global_variable(constary.type, name="_cudapy_cmem",
+                                      addrspace=addrspace)
+        gv.linkage = lc.LINKAGE_INTERNAL
+        gv.global_constant = True
+        gv.initializer = constary
+
+        # Preserve the underlying alignment
+        lldtype = self.get_data_type(aryty.dtype)
+        align = self.get_abi_sizeof(lldtype)
+        gv.align = 2 ** (align - 1).bit_length()
+
+        # Convert to generic address-space
+        conv = nvvmutils.insert_addrspace_conv(lmod, Type.int(8), addrspace)
+        addrspaceptr = gv.bitcast(Type.pointer(Type.int(8), addrspace))
+        genptr = builder.call(conv, [addrspaceptr])
+
+        # Create array object
+        ary = self.make_array(aryty)(self, builder)
+        kshape = [self.get_constant(types.intp, s) for s in arr.shape]
+        kstrides = [self.get_constant(types.intp, s) for s in arr.strides]
+        self.populate_array(ary, data=builder.bitcast(genptr, ary.data.type),
+                            shape=cgutils.pack_array(builder, kshape),
+                            strides=cgutils.pack_array(builder, kstrides),
+                            itemsize=ary.itemsize, parent=ary.parent,
+                            meminfo=None)
+
+        return ary._getvalue()
 
     def insert_const_string(self, mod, string):
         """
