@@ -1,10 +1,9 @@
-from __future__ import print_function, absolute_import
 import numpy as np
 from numba import cuda
-from numba.cuda.testing import unittest, SerialMixin
+from numba.cuda.testing import unittest, CUDATestCase
 
 
-class TestCudaAutoContext(SerialMixin, unittest.TestCase):
+class TestCudaAutoContext(CUDATestCase):
     def test_auto_context(self):
         """A problem was revealed by a customer that the use cuda.to_device
         does not create a CUDA context.
