@@ -3,13 +3,12 @@ import ctypes
 import numpy as np
 
 from numba.cuda.cudadrv import driver, drvapi, devices
-from numba.cuda.testing import unittest, CUDATestCase
-from numba.utils import IS_PY3
+from numba.cuda.testing import unittest, ContextResettingTestCase
 from numba.cuda.testing import skip_on_cudasim
 
 
 @skip_on_cudasim('CUDA Memory API unsupported in the simulator')
-class TestCudaMemory(CUDATestCase):
+class TestCudaMemory(ContextResettingTestCase):
     def setUp(self):
         self.context = devices.get_context()
 
@@ -37,8 +36,7 @@ class TestCudaMemory(CUDATestCase):
 
     def test_pinned_memory(self):
         ary = np.arange(10)
-        arybuf = ary if IS_PY3 else buffer(ary)
-        devmem = self.context.mempin(arybuf, ary.ctypes.data,
+        devmem = self.context.mempin(ary, ary.ctypes.data,
                                      ary.size * ary.dtype.itemsize,
                                      mapped=True)
         self._template(devmem)
@@ -94,7 +92,7 @@ class TestCudaMemory(CUDATestCase):
 
 
 @skip_on_cudasim('CUDA Memory API unsupported in the simulator')
-class TestCudaMemoryFunctions(CUDATestCase):
+class TestCudaMemoryFunctions(ContextResettingTestCase):
     def setUp(self):
         self.context = devices.get_context()
 
@@ -139,7 +137,7 @@ class TestCudaMemoryFunctions(CUDATestCase):
 
 
 @skip_on_cudasim('CUDA Memory API unsupported in the simulator')
-class TestMVExtent(CUDATestCase):
+class TestMVExtent(ContextResettingTestCase):
     def test_c_contiguous_array(self):
         ary = np.arange(100)
         arysz = ary.dtype.itemsize * ary.size
