@@ -1,10 +1,9 @@
-from __future__ import absolute_import, print_function
 import numpy as np
 from numba import cuda
-from numba.cuda.testing import unittest, SerialMixin
+from numba.cuda.testing import unittest, CUDATestCase
 
 
-class TestCudaEvent(SerialMixin, unittest.TestCase):
+class TestCudaEvent(CUDATestCase):
     def test_event_elapsed(self):
         N = 32
         dary = cuda.device_array(N, dtype=np.double)
