@@ -1,10 +1,9 @@
-from __future__ import print_function, division, absolute_import
 import numpy as np
 from numba import cuda, float32, float64, int32
-from numba.cuda.testing import unittest, SerialMixin
+from numba.cuda.testing import unittest, CUDATestCase
 
 
-class TestCudaIDiv(SerialMixin, unittest.TestCase):
+class TestCudaIDiv(CUDATestCase):
     def test_inplace_div(self):
 
         @cuda.jit(argtypes=[float32[:, :], int32, int32])
@@ -15,7 +14,7 @@ class TestCudaIDiv(SerialMixin, unittest.TestCase):
 
         x = np.ones((2, 2), dtype=np.float32)
         grid = cuda.to_device(x)
-        div(grid, 2, 2)
+        div[1, 1](grid, 2, 2)
         y = grid.copy_to_host()
         self.assertTrue(np.all(y == 0.5))
 
@@ -30,7 +29,7 @@ class TestCudaIDiv(SerialMixin, unittest.TestCase):
 
         x = np.ones((2, 2), dtype=np.float64)
         grid = cuda.to_device(x)
-        div_double(grid, 2, 2)
+        div_double[1, 1](grid, 2, 2)
         y = grid.copy_to_host()
         self.assertTrue(np.all(y == 0.5))
 
