@@ -1,10 +1,9 @@
-from __future__ import print_function, absolute_import
 import numpy as np
 from numba import cuda, float32, int32
-from numba.cuda.testing import unittest
+from numba.cuda.testing import unittest, CUDATestCase
 
 
-class TestCudaPy2Div(unittest.TestCase):
+class TestCudaPy2Div(CUDATestCase):
     def test_py2_div_issue(self):
         @cuda.jit(argtypes=[float32[:], float32[:], float32[:], int32])
         def preCalc(y, yA, yB, numDataPoints):
