@@ -1,15 +1,13 @@
-from __future__ import print_function, absolute_import, division
-
 import cmath
 import itertools
 import math
 import sys
 
-from numba import unittest_support as unittest
-from numba.compiler import compile_isolated, Flags, utils
-from numba import types
-from .support import TestCase, tag
+from numba.core.compiler import compile_isolated, Flags, utils
+from numba.core import types
+from numba.tests.support import TestCase, tag
 from .complex_usecases import *
+import unittest
 
 enable_pyobj_flags = Flags()
 enable_pyobj_flags.set("enable_pyobject")
@@ -122,7 +120,6 @@ class TestComplex(BaseComplexTest, TestCase):
                        (types.complex64, types.complex64)]
         self.run_binary(div_usecase, value_types, values, flags=flags)
 
-    @tag('important')
     def test_div_npm(self):
         self.test_div(flags=no_pyobj_flags)
 
@@ -177,22 +174,18 @@ class TestCMath(BaseComplexTest, TestCase):
     def test_isnan(self, flags=enable_pyobj_flags):
         self.check_predicate_func(isnan_usecase, enable_pyobj_flags)
 
-    @tag('important')
     def test_isnan_npm(self):
         self.check_predicate_func(isnan_usecase, no_pyobj_flags)
 
     def test_isinf(self, flags=enable_pyobj_flags):
         self.check_predicate_func(isinf_usecase, enable_pyobj_flags)
 
-    @tag('important')
     def test_isinf_npm(self):
         self.check_predicate_func(isinf_usecase, no_pyobj_flags)
 
-    @unittest.skipIf(utils.PYVERSION < (3, 2), "needs Python 3.2+")
     def test_isfinite(self, flags=enable_pyobj_flags):
         self.check_predicate_func(isfinite_usecase, enable_pyobj_flags)
 
-    @unittest.skipIf(utils.PYVERSION < (3, 2), "needs Python 3.2+")
     def test_isfinite_npm(self):
         self.check_predicate_func(isfinite_usecase, no_pyobj_flags)
 
