@@ -1,14 +1,13 @@
-from __future__ import print_function
-
 import sys
 import numpy as np
 
-import numba.unittest_support as unittest
-from numba.compiler import compile_isolated, Flags
-from numba import jit, njit, types
-from .support import TestCase, MemoryLeakMixin, tag
+import unittest
+from numba.core.compiler import compile_isolated, Flags
+from numba import jit, njit
+from numba.core import types
+from numba.tests.support import TestCase, MemoryLeakMixin
 from numba import testing
-from numba.datamodel.testing import test_factory
+from numba.core.datamodel.testing import test_factory
 
 
 enable_pyobj_flags = Flags()
@@ -148,7 +147,6 @@ class TestGenerators(MemoryLeakMixin, TestCase):
         cgen = cr.entry_point(8)
         self.check_generator(pygen, cgen)
 
-    @tag('important')
     def test_gen1(self):
         self.check_gen1()
 
@@ -162,7 +160,6 @@ class TestGenerators(MemoryLeakMixin, TestCase):
         cgen = cr.entry_point(8)
         self.check_generator(pygen, cgen)
 
-    @tag('important')
     def test_gen2(self):
         self.check_gen2()
 
@@ -176,7 +173,6 @@ class TestGenerators(MemoryLeakMixin, TestCase):
         cgen = cr.entry_point(8)
         self.check_generator(pygen, cgen)
 
-    @tag('important')
     def test_gen3(self):
         self.check_gen3()
 
@@ -190,7 +186,6 @@ class TestGenerators(MemoryLeakMixin, TestCase):
         cgen = cr.entry_point(5, 6, 7)
         self.check_generator(pygen, cgen)
 
-    @tag('important')
     def test_gen4(self):
         self.check_gen4()
 
@@ -219,7 +214,6 @@ class TestGenerators(MemoryLeakMixin, TestCase):
             l.append(next(cgen))
         self.assertEqual(l, [14] * 3)
 
-    @tag('important')
     def test_gen6(self):
         self.check_gen6()
 
@@ -235,7 +229,6 @@ class TestGenerators(MemoryLeakMixin, TestCase):
         cgen = cr.entry_point(arr)
         self.check_generator(pygen, cgen)
 
-    @tag('important')
     def test_gen7(self):
         self.check_gen7()
 
@@ -255,7 +248,6 @@ class TestGenerators(MemoryLeakMixin, TestCase):
         check(y=5)
         check(x=6, b=True)
 
-    @tag('important')
     def test_gen8(self):
         self.check_gen8(nopython=True)
 
@@ -269,7 +261,6 @@ class TestGenerators(MemoryLeakMixin, TestCase):
         cgen = cr.entry_point()
         self.check_generator(pygen, cgen)
 
-    @tag('important')
     def test_gen9(self):
         self.check_gen9(flags=no_pyobj_flags)
 
@@ -290,7 +281,6 @@ class TestGenerators(MemoryLeakMixin, TestCase):
     def test_consume_gen2(self):
         self.check_consume_generator(gen2)
 
-    @tag('important')
     def test_consume_gen3(self):
         self.check_consume_generator(gen3)
 
