@@ -1,5 +1,3 @@
-from __future__ import division, print_function
-
 import contextlib
 import gc
 from itertools import product, cycle
@@ -10,22 +8,12 @@ import platform
 
 import numpy as np
 
-from numba import unittest_support as unittest
-from numba import jit, errors
-from numba.numpy_support import version as numpy_version
-from .support import TestCase, tag
-from .matmul_usecase import matmul_usecase, needs_matmul, needs_blas
-
-_is_armv7l = platform.machine() == 'armv7l'
-
-try:
-    import scipy.linalg.cython_lapack
-    has_lapack = True
-except ImportError:
-    has_lapack = False
-
-needs_lapack = unittest.skipUnless(has_lapack,
-                                   "LAPACK needs Scipy 0.16+")
+from numba import jit
+from numba.core import errors
+from numba.tests.support import (TestCase, tag, needs_lapack, needs_blas,
+                                 _is_armv7l, skip_ppc64le_issue4026)
+from .matmul_usecase import matmul_usecase
+import unittest
 
 
 def dot2(a, b):
@@ -131,7 +119,6 @@ class TestProduct(TestCase):
                       % (func_name,),
                       str(raises.exception))
 
-    @needs_blas
     def check_dot_vv(self, pyfunc, func_name):
         n = 3
         cfunc = jit(nopython=True)(pyfunc)
@@ -151,19 +138,20 @@ class TestProduct(TestCase):
         b = self.sample_vector(n, np.float64)
         self.assert_mismatching_dtypes(cfunc, (a, b), func_name=func_name)
 
+    @needs_blas
     def test_dot_vv(self):
         """
         Test vector * vector np.dot()
         """
         self.check_dot_vv(dot2, "np.dot()")
 
+    @needs_blas
     def test_vdot(self):
         """
         Test np.vdot()
         """
         self.check_dot_vv(vdot, "np.vdot()")
 
-    @needs_blas
     def check_dot_vm(self, pyfunc2, pyfunc3, func_name):
         m, n = 2, 3
 
@@ -215,13 +203,13 @@ class TestProduct(TestCase):
             out = np.empty(m, np.float32)
             self.assert_mismatching_dtypes(cfunc3, (a, b, out), func_name)
 
+    @needs_blas
     def test_dot_vm(self):
         """
         Test vector * matrix and matrix * vector np.dot()
         """
         self.check_dot_vm(dot2, dot3, "np.dot()")
 
-    @needs_blas
     def check_dot_mm(self, pyfunc2, pyfunc3, func_name):
 
         def samples(m, n, k):
@@ -279,28 +267,28 @@ class TestProduct(TestCase):
             out = np.empty((m, n), np.float32)
             self.assert_mismatching_dtypes(cfunc3, (a, b, out), func_name)
 
-    @tag('important')
+    @needs_blas
     def test_dot_mm(self):
         """
         Test matrix * matrix np.dot()
         """
         self.check_dot_mm(dot2, dot3, "np.dot()")
 
-    @needs_matmul
+    @needs_blas
     def test_matmul_vv(self):
         """
         Test vector @ vector
         """
         self.check_dot_vv(matmul_usecase, "'@'")
 
-    @needs_matmul
+    @needs_blas
     def test_matmul_vm(self):
         """
         Test vector @ matrix and matrix @ vector
         """
         self.check_dot_vm(matmul_usecase, None, "'@'")
 
-    @needs_matmul
+    @needs_blas
     def test_matmul_mm(self):
         """
         Test matrix @ matrix
@@ -408,12 +396,8 @@ def trace_matrix_no_offset(a):
     return np.trace(a)
 
 
-if numpy_version >= (1, 9):
-    def outer_matrix(a, b, out=None):
-        return np.outer(a, b, out=out)
-else:
-    def outer_matrix(a, b):
-        return np.outer(a, b)
+def outer_matrix(a, b, out=None):
+    return np.outer(a, b, out=out)
 
 
 def kron_matrix(a, b):
@@ -525,20 +509,6 @@ class TestLinalgBase(TestCase):
 
         return Q
 
-    def shape_with_0_input(self, *args):
-        """
-        returns True if an input argument has a dimension that is zero
-        and Numpy version is < 1.13, else False. This is due to behaviour
-        changes in handling dimension zero arrays:
-        https://github.com/numpy/numpy/issues/10573
-        """
-        if numpy_version < (1, 13):
-            for x in args:
-                if isinstance(x, np.ndarray):
-                    if 0 in x.shape:
-                        return True
-        return False
-
     def assert_error(self, cfunc, args, msg, err=ValueError):
         with self.assertRaises(err) as raises:
             cfunc(*args)
@@ -741,7 +711,6 @@ class TestLinalgInv(TestLinalgBase):
     Tests for np.linalg.inv.
     """
 
-    @tag('important')
     @needs_lapack
     def test_linalg_inv(self):
         """
@@ -794,6 +763,7 @@ class TestLinalgInv(TestLinalgBase):
         self.assert_raise_on_singular(cfunc, (np.zeros((2, 2)),))
 
 
+@skip_ppc64le_issue4026
 class TestLinalgCholesky(TestLinalgBase):
     """
     Tests for np.linalg.cholesky.
@@ -824,11 +794,6 @@ class TestLinalgCholesky(TestLinalgBase):
         cfunc = jit(nopython=True)(cholesky_matrix)
 
         def check(a):
-            if self.shape_with_0_input(a):
-                # has shape with 0 on input, numpy will fail,
-                # just make sure Numba runs without error
-                cfunc(a)
-                return
             expected = cholesky_matrix(a)
             got = cfunc(a)
             use_reconstruction = False
@@ -911,11 +876,6 @@ class TestLinalgEigenSystems(TestLinalgBase):
         cfunc = jit(nopython=True)(func)
 
         def check(a):
-            if self.shape_with_0_input(a):
-                # has shape with 0 on input, numpy will fail,
-                # just make sure Numba runs without error
-                cfunc(a)
-                return
             expected = func(a)
             got = cfunc(a)
             # check that the returned tuple is same length
@@ -1671,11 +1631,6 @@ class TestLinalgPinv(TestLinalgBase):
         cfunc = jit(nopython=True)(pinv_matrix)
 
         def check(a, **kwargs):
-            if self.shape_with_0_input(a):
-                # has shape with 0 on input, numpy will fail,
-                # just make sure Numba runs without error
-                cfunc(a, **kwargs)
-                return
             expected = pinv_matrix(a, **kwargs)
             got = cfunc(a, **kwargs)
 
@@ -1794,11 +1749,6 @@ class TestLinalgDetAndSlogdet(TestLinalgBase):
     """
 
     def check_det(self, cfunc, a, **kwargs):
-        if self.shape_with_0_input(a):
-            # has shape with 0 on input, numpy will fail,
-            # just make sure Numba runs without error
-            cfunc(a, **kwargs)
-            return
         expected = det_matrix(a, **kwargs)
         got = cfunc(a, **kwargs)
 
@@ -1812,11 +1762,6 @@ class TestLinalgDetAndSlogdet(TestLinalgBase):
             cfunc(a, **kwargs)
 
     def check_slogdet(self, cfunc, a, **kwargs):
-        if self.shape_with_0_input(a):
-            # has shape with 0 on input, numpy will fail,
-            # just make sure Numba runs without error
-            cfunc(a, **kwargs)
-            return
         expected = slogdet_matrix(a, **kwargs)
         got = cfunc(a, **kwargs)
 
@@ -2053,26 +1998,17 @@ class TestLinalgCond(TestLinalgBase):
             self.assert_raise_on_empty(cfunc, (np.empty(sz),))
 
         # singular systems to trip divide-by-zero
-        # only for np > 1.14, before this norm was computed via inversion which
-        # will fail with numpy.linalg.linalg.LinAlgError: Singular matrix
-        if numpy_version > (1, 14):
-            x = np.array([[1, 0], [0, 0]], dtype=np.float64)
-            check(x)
-            check(x, p=2)
-            x = np.array([[0, 0], [0, 0]], dtype=np.float64)
-            check(x, p=-2)
+        x = np.array([[1, 0], [0, 0]], dtype=np.float64)
+        check(x)
+        check(x, p=2)
+        x = np.array([[0, 0], [0, 0]], dtype=np.float64)
+        check(x, p=-2)
 
         # try an ill-conditioned system with 2-norm, make sure np raises an
         # overflow warning as the result is `+inf` and that the result from
         # numba matches.
         with warnings.catch_warnings():
             a = np.array([[1.e308, 0], [0, 0.1]], dtype=np.float64)
-            if numpy_version < (1, 15):
-                # overflow warning is silenced in np >= 1.15
-                warnings.simplefilter("error", RuntimeWarning)
-                self.assertRaisesRegexp(RuntimeWarning,
-                                        'overflow encountered in.*',
-                                        check, a)
             warnings.simplefilter("ignore", RuntimeWarning)
             check(a)
 
@@ -2419,10 +2355,9 @@ class TestBasics(TestLinalgSystems):  # TestLinalgSystems for 1d test
                 product(self.sizes, self.sizes, self.dtypes):
             (a, b) = self._get_input(size1, size2, dtype)
             check(a, b)
-            if numpy_version >= (1, 9):
-                c = np.empty((np.asarray(a).size, np.asarray(b).size),
-                             dtype=np.asarray(a).dtype)
-                check(a, b, out=c)
+            c = np.empty((np.asarray(a).size, np.asarray(b).size),
+                            dtype=np.asarray(a).dtype)
+            check(a, b, out=c)
 
         self._assert_wrong_dim("outer", cfunc)
 
