@@ -10,17 +10,17 @@ test for getitem assumes makes use of these three operations and therefore
 assumes that they work.
 
 """
-from __future__ import print_function, absolute_import, division
+
+from textwrap import dedent
 
 from numba import njit
-from numba import int32, types
-from numba.errors import TypingError
-from numba import listobject
-from numba.utils import IS_PY3
-from .support import (TestCase, MemoryLeakMixin, unittest, override_config,
-                      forbid_codegen)
-
-skip_py2 = unittest.skipUnless(IS_PY3, reason='not supported in py2')
+from numba import int32
+from numba.extending import register_jitable
+from numba.core import types
+from numba.core.errors import TypingError
+from numba.tests.support import (TestCase, MemoryLeakMixin, override_config,
+                                 forbid_codegen)
+from numba.typed import listobject
 
 
 class TestCreateAppendLength(MemoryLeakMixin, TestCase):
@@ -443,6 +443,7 @@ class TestGetitemSlice(MemoryLeakMixin, TestCase):
 
     def test_list_getitem_multiple_slice_zero_step_index_error(self):
         self.disable_leak_check()
+
         @njit
         def foo():
             l = listobject.new_list(int32)
@@ -654,6 +655,7 @@ class TestPop(MemoryLeakMixin, TestCase):
 
     def test_list_pop_empty_index_error_with_index(self):
         self.disable_leak_check()
+
         @njit
         def foo(i):
             l = listobject.new_list(int32)
@@ -682,6 +684,7 @@ class TestPop(MemoryLeakMixin, TestCase):
 
     def test_list_pop_mutiple_index_error_with_index(self):
         self.disable_leak_check()
+
         @njit
         def foo(i):
             l = listobject.new_list(int32)
@@ -923,6 +926,7 @@ class TestExtend(MemoryLeakMixin, TestCase):
 
     def test_list_extend_typing_error_non_iterable(self):
         self.disable_leak_check()
+
         @njit
         def foo():
             l = listobject.new_list(int32)
@@ -1003,6 +1007,7 @@ class TestInsert(MemoryLeakMixin, TestCase):
 
     def test_list_insert_typing_error(self):
         self.disable_leak_check()
+
         @njit
         def foo():
             l = listobject.new_list(int32)
@@ -1021,6 +1026,7 @@ class TestRemove(MemoryLeakMixin, TestCase):
 
     def test_list_remove_empty(self):
         self.disable_leak_check()
+
         @njit
         def foo():
             l = listobject.new_list(int32)
@@ -1041,6 +1047,7 @@ class TestRemove(MemoryLeakMixin, TestCase):
 
     def test_list_remove_singleton_value_error(self):
         self.disable_leak_check()
+
         @njit
         def foo():
             l = listobject.new_list(int32)
@@ -1064,6 +1071,7 @@ class TestRemove(MemoryLeakMixin, TestCase):
 
     def test_list_remove_multiple_value_error(self):
         self.disable_leak_check()
+
         @njit
         def foo():
             l = listobject.new_list(int32)
@@ -1220,6 +1228,7 @@ class TestIndex(MemoryLeakMixin, TestCase):
 
     def test_index_singleton_value_error(self):
         self.disable_leak_check()
+
         @njit
         def foo():
             l = listobject.new_list(int32)
@@ -1235,6 +1244,7 @@ class TestIndex(MemoryLeakMixin, TestCase):
 
     def test_index_multiple_value_error(self):
         self.disable_leak_check()
+
         @njit
         def foo():
             l = listobject.new_list(int32)
@@ -1251,6 +1261,7 @@ class TestIndex(MemoryLeakMixin, TestCase):
 
     def test_index_multiple_value_error_start(self):
         self.disable_leak_check()
+
         @njit
         def foo(start):
             l = listobject.new_list(int32)
@@ -1269,6 +1280,7 @@ class TestIndex(MemoryLeakMixin, TestCase):
 
     def test_index_multiple_value_error_end(self):
         self.disable_leak_check()
+
         @njit
         def foo(end):
             l = listobject.new_list(int32)
@@ -1287,6 +1299,7 @@ class TestIndex(MemoryLeakMixin, TestCase):
 
     def test_index_typing_error_start(self):
         self.disable_leak_check()
+
         @njit
         def foo():
             l = listobject.new_list(int32)
@@ -1302,6 +1315,7 @@ class TestIndex(MemoryLeakMixin, TestCase):
 
     def test_index_typing_error_end(self):
         self.disable_leak_check()
+
         @njit
         def foo():
             l = listobject.new_list(int32)
@@ -1428,7 +1442,6 @@ class TestIter(MemoryLeakMixin, TestCase):
 class TestStringItem(MemoryLeakMixin, TestCase):
     """Test list can take strings as items. """
 
-    @skip_py2
     def test_string_item(self):
         @njit
         def foo():
@@ -1479,7 +1492,6 @@ class TestItemCasting(TestCase):
         self.check_good(types.boolean, types.float64)
         self.check_good(types.boolean, types.complex128)
 
-    @skip_py2
     def test_cast_fail_unicode_int(self):
 
         @njit
@@ -1494,7 +1506,6 @@ class TestItemCasting(TestCase):
             str(raises.exception),
         )
 
-    @skip_py2
     def test_cast_fail_int_unicode(self):
 
         @njit
@@ -1508,3 +1519,98 @@ class TestItemCasting(TestCase):
             'Cannot cast int32 to unicode_type',
             str(raises.exception),
         )
+
+
+@register_jitable
+def make_test_list():
+    l = listobject.new_list(int32)
+    l.append(int32(1))
+    return l
+
+
+class TestImmutable(MemoryLeakMixin, TestCase):
+
+    def test_is_immutable(self):
+        @njit
+        def foo():
+            l = make_test_list()
+            return l._is_mutable()
+        self.assertTrue(foo())
+
+    def test_make_immutable_is_immutable(self):
+        @njit
+        def foo():
+            l = make_test_list()
+            l._make_immutable()
+            return l._is_mutable()
+        self.assertFalse(foo())
+
+    def test_length_still_works_when_immutable(self):
+        @njit
+        def foo():
+            l = make_test_list()
+            l._make_immutable()
+            return len(l),l._is_mutable()
+        length, mutable = foo()
+        self.assertEqual(length, 1)
+        self.assertFalse(mutable)
+
+    def test_getitem_still_works_when_immutable(self):
+        @njit
+        def foo():
+            l = make_test_list()
+            l._make_immutable()
+            return l[0], l._is_mutable()
+        test_item, mutable = foo()
+        self.assertEqual(test_item, 1)
+        self.assertFalse(mutable)
+
+    def test_append_fails(self):
+        self.disable_leak_check()
+
+        @njit
+        def foo():
+            l = make_test_list()
+            l._make_immutable()
+            l.append(int32(1))
+        with self.assertRaises(ValueError) as raises:
+            foo()
+        self.assertIn(
+            'list is immutable',
+            str(raises.exception),
+        )
+
+    def test_mutation_fails(self):
+        """ Test that any attempt to mutate an immutable typed list fails. """
+        self.disable_leak_check()
+
+        def generate_function(line):
+            context = {}
+            exec(dedent("""
+                from numba.typed import listobject
+                from numba import int32
+                def bar():
+                    lst = listobject.new_list(int32)
+                    lst.append(int32(1))
+                    lst._make_immutable()
+                    zero = int32(0)
+                    {}
+                """.format(line)), context)
+            return njit(context["bar"])
+        for line in ("lst.append(zero)",
+                     "lst[0] = zero",
+                     "lst.pop()",
+                     "del lst[0]",
+                     "lst.extend((zero,))",
+                     "lst.insert(0, zero)",
+                     "lst.clear()",
+                     "lst.reverse()",
+                     "lst.sort()",
+                     ):
+            foo = generate_function(line)
+            with self.assertRaises(ValueError) as raises:
+                foo()
+            self.assertIn(
+                "list is immutable",
+                str(raises.exception),
+            )
