@@ -1,17 +1,16 @@
-from __future__ import print_function, division, absolute_import
-
 import os, sys, subprocess
 import itertools
 
 import numpy as np
 
-from numba import unittest_support as unittest
-from numba.compiler import compile_isolated
-from numba import types, typeinfer, typing, jit, errors
-from numba.typeconv import Conversion
+from numba.core.compiler import compile_isolated
+from numba import jit
+from numba.core import types, typing, errors, typeinfer, utils
+from numba.core.typeconv import Conversion
 
-from .support import TestCase, tag
-from .test_typeconv import CompatibilityTestMixin
+from numba.tests.support import TestCase, tag
+from numba.tests.test_typeconv import CompatibilityTestMixin
+import unittest
 
 
 i8 = types.int8
@@ -132,7 +131,6 @@ class TestUnify(unittest.TestCase):
     def assert_unify_failure(self, aty, bty):
         self.assert_unify(aty, bty, None)
 
-    @tag('important')
     def test_integer(self):
         ctx = typing.Context()
         for aty, bty in itertools.product(types.integer_domain,
@@ -144,7 +142,6 @@ class TestUnify(unittest.TestCase):
                 expected = self.int_unify[key[::-1]]
             self.assert_unify(aty, bty, getattr(types, expected))
 
-    @tag('important')
     def test_bool(self):
         aty = types.boolean
         for bty in types.integer_domain:
@@ -188,7 +185,6 @@ class TestUnify(unittest.TestCase):
             for res in results:
                 self.assertEqual(res, expected)
 
-    @tag('important')
     def test_none(self):
         aty = types.none
         bty = types.none
@@ -209,7 +205,6 @@ class TestUnify(unittest.TestCase):
         bty = types.Optional(types.slice3_type)
         self.assert_unify_failure(aty, bty)
 
-    @tag('important')
     def test_tuple(self):
         aty = types.UniTuple(i32, 3)
         bty = types.UniTuple(i64, 3)
@@ -269,7 +264,6 @@ class TestUnify(unittest.TestCase):
         self.assert_unify(aty, bty, types.Tuple((types.Optional(i32),
                                                  types.Optional(i64))))
 
-    @tag('important')
     def test_arrays(self):
         aty = types.Array(i32, 3, "C")
         bty = types.Array(i32, 3, "A")
@@ -292,7 +286,6 @@ class TestUnify(unittest.TestCase):
         bty = types.Array(u32, 2, "C")
         self.assert_unify_failure(aty, bty)
 
-    @tag('important')
     def test_list(self):
         aty = types.List(types.undefined)
         bty = types.List(i32)
@@ -325,7 +318,6 @@ class TestUnify(unittest.TestCase):
         bty = types.Set(types.Tuple([i16]))
         self.assert_unify_failure(aty, bty)
 
-    @tag('important')
     def test_range(self):
         aty = types.range_state32_type
         bty = types.range_state64_type
@@ -347,7 +339,6 @@ class TestTypeConversion(CompatibilityTestMixin, unittest.TestCase):
         got = ctx.can_convert(aty, bty)
         self.assertIsNone(got)
 
-    @tag('important')
     def test_convert_number_types(self):
         # Check that Context.can_convert() is compatible with the default
         # number conversion rules registered in the typeconv module
@@ -355,7 +346,6 @@ class TestTypeConversion(CompatibilityTestMixin, unittest.TestCase):
         ctx = typing.Context()
         self.check_number_compatibility(ctx.can_convert)
 
-    @tag('important')
     def test_tuple(self):
         # UniTuple -> UniTuple
         aty = types.UniTuple(i32, 3)
@@ -389,7 +379,6 @@ class TestTypeConversion(CompatibilityTestMixin, unittest.TestCase):
         aty = types.UniTuple(i64, 2)
         bty = types.UniTuple(i64, 3)
 
-    @tag('important')
     def test_arrays(self):
         # Different layouts
         aty = types.Array(i32, 3, "C")
@@ -514,7 +503,6 @@ class TestUnifyUseCases(unittest.TestCase):
         cres = compile_isolated(pyfunc, argtys)
         return (pyfunc, cres)
 
-    @tag('important')
     def test_complex_unify_issue599(self):
         pyfunc, cres = self._actually_test_complex_unify()
         arg = np.array([1.0j])
@@ -536,7 +524,6 @@ class TestUnifyUseCases(unittest.TestCase):
             subproc.wait()
             self.assertEqual(subproc.returncode, 0, 'Child process failed.')
 
-    @tag('important')
     def test_int_tuple_unify(self):
         """
         Test issue #493
@@ -642,7 +629,6 @@ def issue_1394(a):
 
 class TestMiscIssues(TestCase):
 
-    @tag('important')
     def test_issue_797(self):
         """https://github.com/numba/numba/issues/797#issuecomment-58592401
 
@@ -652,7 +638,6 @@ class TestMiscIssues(TestCase):
         g = np.zeros(shape=(10, 10), dtype=np.int32)
         foo(np.int32(0), np.int32(0), np.int32(1), np.int32(1), g)
 
-    @tag('important')
     def test_issue_1080(self):
         """https://github.com/numba/numba/issues/1080
 
@@ -661,7 +646,6 @@ class TestMiscIssues(TestCase):
         foo = jit(nopython=True)(issue_1080)
         foo(True, False)
 
-    @tag('important')
     def test_list_unify1(self):
         """
         Exercise back-propagation of refined list type.
@@ -672,7 +656,6 @@ class TestMiscIssues(TestCase):
             res = cfunc(n)
             self.assertPreciseEqual(res, pyfunc(n))
 
-    @tag('important')
     def test_list_unify2(self):
         pyfunc = list_unify_usecase2
         cfunc = jit(nopython=True)(pyfunc)
@@ -681,7 +664,6 @@ class TestMiscIssues(TestCase):
         # converted values).
         self.assertEqual(res, pyfunc(3))
 
-    @tag('important')
     def test_range_unify(self):
         pyfunc = range_unify_usecase
         cfunc = jit(nopython=True)(pyfunc)
@@ -689,7 +671,6 @@ class TestMiscIssues(TestCase):
             res = cfunc(v)
             self.assertPreciseEqual(res, pyfunc(v))
 
-    @tag('important')
     def test_issue_1394(self):
         pyfunc = issue_1394
         cfunc = jit(nopython=True)(pyfunc)
@@ -698,5 +679,60 @@ class TestMiscIssues(TestCase):
             self.assertEqual(res, pyfunc(v))
 
 
+class TestFoldArguments(unittest.TestCase):
+    def check_fold_arguments_list_inputs(self, func, args, kws):
+        def make_tuple(*args):
+            return args
+
+        unused_handler = None
+
+        pysig = utils.pysignature(func)
+        names = list(pysig.parameters)
+
+        with self.subTest(kind='dict'):
+            folded_dict = typing.fold_arguments(
+                pysig, args, kws, make_tuple, unused_handler, unused_handler,
+            )
+            # correct ordering
+            for i, (j, k) in enumerate(zip(folded_dict, names)):
+                (got_index, got_param, got_name) = j
+                self.assertEqual(got_index, i)
+                self.assertEqual(got_name, f'arg.{k}')
+
+        kws = list(kws.items())
+        with self.subTest(kind='list'):
+            folded_list = typing.fold_arguments(
+                pysig, args, kws, make_tuple, unused_handler, unused_handler,
+            )
+            self.assertEqual(folded_list, folded_dict)
+
+    def test_fold_arguments_list_inputs(self):
+        cases = [
+            dict(
+                func=lambda a, b, c, d: None,
+                args=['arg.a', 'arg.b'],
+                kws=dict(c='arg.c', d='arg.d')
+            ),
+            dict(
+                func=lambda: None,
+                args=[],
+                kws=dict(),
+            ),
+            dict(
+                func=lambda a: None,
+                args=['arg.a'],
+                kws={},
+            ),
+            dict(
+                func=lambda a: None,
+                args=[],
+                kws=dict(a='arg.a'),
+            ),
+        ]
+        for case in cases:
+            with self.subTest(**case):
+                self.check_fold_arguments_list_inputs(**case)
+
+
 if __name__ == '__main__':
     unittest.main()
