@@ -2,7 +2,6 @@
 Tests for numba.types.
 """
 
-from __future__ import print_function, absolute_import
 
 from collections import namedtuple
 import contextlib
@@ -13,24 +12,27 @@ import operator
 import shutil
 import sys
 import uuid
-try:
-    import cPickle as pickle
-except ImportError:
-    import pickle
 import weakref
 
 import numpy as np
 
-from numba import unittest_support as unittest
-from numba.utils import IS_PY3
-from numba import sigutils, types, typing, errors
-from numba.types.abstract import _typecache
-from numba.typing.templates import make_overload_template
-from numba import jit, njit, numpy_support, typeof
-from numba.extending import (overload, register_model, models, unbox,
-                             NativeValue, typeof_impl)
-from .support import TestCase, tag, temp_directory
-from .enum_usecases import Color, Shake, Shape
+from numba.core import types, typing, errors, sigutils
+from numba.core.types.abstract import _typecache
+from numba.core.types.functions import _header_lead
+from numba.core.typing.templates import make_overload_template
+from numba import jit, njit, typeof
+from numba.core.extending import (overload, register_model, models, unbox,
+                                  NativeValue, typeof_impl)
+from numba.tests.support import TestCase, temp_directory
+from numba.tests.enum_usecases import Color, Shake, Shape
+import unittest
+from numba.np import numpy_support
+
+
+try:
+    import cPickle as pickle
+except ImportError:
+    import pickle
 
 
 Point = namedtuple('Point', ('x', 'y'))
@@ -48,7 +50,6 @@ class Dummy(object):
 
 class TestTypes(TestCase):
 
-    @tag('important')
     def test_equality(self):
         self.assertEqual(types.int32, types.int32)
         self.assertEqual(types.uint32, types.uint32)
@@ -140,7 +141,6 @@ class TestTypes(TestCase):
         self.assertTrue(b != c)
         self.assertTrue(a != z)
 
-    @tag('important')
     def test_interning(self):
         # Test interning and lifetime of dynamic types.
         a = types.Dummy('xyzzyx')
@@ -176,12 +176,19 @@ class TestTypes(TestCase):
         gc.collect()
         self.assertEqual(len(cache), cache_len)
 
-    @tag('important')
     def test_array_notation(self):
         def check(arrty, scalar, ndim, layout):
             self.assertIs(arrty.dtype, scalar)
             self.assertEqual(arrty.ndim, ndim)
             self.assertEqual(arrty.layout, layout)
+
+        def check_index_error(callable):
+            with self.assertRaises(KeyError) as raises:
+                callable()
+            self.assertIn(
+                "Can only index numba types with slices with no start or "
+                "stop, got", str(raises.exception))
+
         scalar = types.int32
         check(scalar[:], scalar, 1, 'A')
         check(scalar[::1], scalar, 1, 'C')
@@ -189,6 +196,12 @@ class TestTypes(TestCase):
         check(scalar[:, ::1], scalar, 2, 'C')
         check(scalar[::1, :], scalar, 2, 'F')
 
+        check_index_error(lambda: scalar[0])
+        check_index_error(lambda: scalar[:, 4])
+        check_index_error(lambda: scalar[::1, 1:])
+        check_index_error(lambda: scalar[:2])
+        check_index_error(lambda: list(scalar))
+
     def test_array_notation_for_dtype(self):
         def check(arrty, scalar, ndim, layout):
             self.assertIs(arrty.dtype, scalar)
@@ -202,7 +215,6 @@ class TestTypes(TestCase):
         check(dtyped[:, ::1], scalar, 2, 'C')
         check(dtyped[::1, :], scalar, 2, 'F')
 
-    @tag('important')
     def test_call_notation(self):
         # Function call signature
         i = types.int32
@@ -297,13 +309,12 @@ class TestNumbers(TestCase):
         check_order([types.float32, types.float64])
         check_order([types.complex64, types.complex128])
 
-        if IS_PY3:
-            with self.assertRaises(TypeError):
-                types.int8 <= types.uint32
-            with self.assertRaises(TypeError):
-                types.int8 <= types.float32
-            with self.assertRaises(TypeError):
-                types.float64 <= types.complex128
+        with self.assertRaises(TypeError):
+            types.int8 <= types.uint32
+        with self.assertRaises(TypeError):
+            types.int8 <= types.float32
+        with self.assertRaises(TypeError):
+            types.float64 <= types.complex128
 
 
 class TestNdIter(TestCase):
@@ -506,8 +517,8 @@ class TestPickling(TestCase):
     # call templates are not picklable
     @unittest.expectedFailure
     def test_external_function_pointers(self):
-        from numba.typing import ctypes_utils
-        from .ctypes_usecases import c_sin, c_cos
+        from numba.core.typing import ctypes_utils
+        from numba.tests.ctypes_usecases import c_sin, c_cos
         for fnptr in (c_sin, c_cos):
             ty = ctypes_utils.make_function_type(fnptr)
             self.assertIsInstance(ty, types.ExternalFunctionPointer)
@@ -622,7 +633,7 @@ class TestIsInternalTypeMarker(TestCase):
     the `is_internal` attr of a concrete Type class
     """
     source_lines = """
-from numba import types
+from numba.core import types
 
 class FooType(types.Type):
     def __init__(self):
@@ -744,7 +755,7 @@ class FooType(types.Type):
             with self.assertRaises(errors.TypingError) as raises:
                 call_false_if_not_array_closed_system(Foo())
             estr = str(raises.exception)
-            self.assertIn("Invalid use of Function", estr)
+            self.assertIn(_header_lead, estr)
             self.assertIn("false_if_not_array_closed_system", estr)
             self.assertIn("(Foo)", estr)
 
@@ -789,5 +800,61 @@ class FooType(types.Type):
                              f(Foo()))
 
 
+class TestIssues(TestCase):
+    def test_omitted_type(self):
+        # issue https://github.com/numba/numba/issues/5471
+        def inner(a):
+            pass
+
+        @overload(inner)
+        def inner_overload(a):
+            if not isinstance(a, types.Literal):
+                return
+            return lambda a: a
+
+        @njit
+        def my_func(a='a'):
+            return inner(a)
+
+        @njit
+        def f():
+            return my_func()
+
+        @njit
+        def g():
+            return my_func('b')
+
+        self.assertEqual(f(), 'a')
+        self.assertEqual(g(), 'b')
+
+    def test_type_of_literal(self):
+        # type(val) where val is a literal should not give a literal type.
+        def inner(a):
+            pass
+
+        @overload(inner)
+        def inner_overload(a):
+            if not isinstance(a, types.Literal):
+                return
+            self.assertIsInstance(a, types.Literal)
+            # NOTE: using 1.23 to ensure that the result is indeed an int.
+            return lambda a: type(a)(a + 1.23)
+
+        @njit
+        def my_func(a=1):
+            return inner(a)
+
+        @njit
+        def f():
+            return my_func()
+
+        @njit
+        def g():
+            return my_func(100)
+
+        self.assertEqual(f(), 2)
+        self.assertEqual(g(), 101)
+
+
 if __name__ == '__main__':
     unittest.main()
