@@ -1,16 +1,15 @@
-from __future__ import print_function
-
 import random
 import numpy as np
 
-from .support import TestCase, captured_stdout
-from numba import njit, types
-from numba.unsafe.tuple import tuple_setitem
-from numba.unsafe.ndarray import to_fixed_tuple, empty_inferred
-from numba.unsafe.bytes import memcpy_region
-from numba.unsafe.refcount import dump_refcount
-from numba.unsafe.numbers import trailing_zeros, leading_zeros
-from numba.errors import TypingError
+from numba.tests.support import TestCase, captured_stdout
+from numba import njit
+from numba.core import types
+from numba.cpython.unsafe.tuple import tuple_setitem
+from numba.np.unsafe.ndarray import to_fixed_tuple, empty_inferred
+from numba.core.unsafe.bytes import memcpy_region
+from numba.core.unsafe.refcount import dump_refcount
+from numba.cpython.unsafe.numbers import trailing_zeros, leading_zeros
+from numba.core.errors import TypingError
 
 
 class TestTupleIntrinsic(TestCase):
@@ -189,7 +188,7 @@ class TestZeroCounts(TestCase):
         unsupported_types = filter(
             lambda x: not isinstance(x, types.Integer), types.number_domain
         )
-        for typ in unsupported_types:
+        for typ in sorted(unsupported_types, key=str):
             with self.assertRaises(TypingError) as e:
                 cfunc(typ(2))
             self.assertIn(
@@ -198,16 +197,18 @@ class TestZeroCounts(TestCase):
                 str(e.exception),
             )
 
-        # Testing w/ too many arguments
-        arg_cases = [(1, 2), ()]
-        for args in arg_cases:
+        # Testing w/ too many/few arguments
+        def check(args, string):
             with self.assertRaises(TypingError) as e:
                 cfunc(*args)
             self.assertIn(
-                "Invalid use of Function({})".format(str(func)),
+                "{}() ".format(func_name),
                 str(e.exception)
             )
 
+        check((1, 2), "takes 2 positional arguments but 3 were given")
+        check((), "missing 1 required positional argument")
+
     def test_trailing_zeros_error(self):
         self.check_error_msg(trailing_zeros)
 
